mod parsing;
mod race_record;

type NumericValue = u64;

pub fn solve_task_1(input: &str) -> String {
    let parsed = parsing::parsing(input);
    let mut how_many_times_won = 1;
    for next_record in parsed {
        how_many_times_won *= all_rounds_of_durations(next_record.time())
            .into_iter()
            .filter(|&distance| distance > next_record.reached_distance())
            .count() as NumericValue;
    }
    how_many_times_won.to_string()
}
pub fn solve_task_2(input: &str) -> String {
    let parsed = parsing::parsing_part_2(input);
    let how_many_times_won = all_rounds_of_durations(parsed.time())
        .into_iter()
        .filter(|&distance| distance > parsed.reached_distance())
        .count() as NumericValue;
    how_many_times_won.to_string()
}

fn all_rounds_of_durations(duration: NumericValue) -> Vec<NumericValue> {
    std::iter::repeat(duration)
        .take(duration as usize)
        .enumerate()
        .map(|(speed_duration, duration)| {
            simuluate_reached_distance_at(duration, speed_duration as NumericValue)
        })
        .collect()
}

fn simuluate_reached_distance_at(
    duration: NumericValue,
    speed_duration: NumericValue,
) -> NumericValue {
    assert!(duration >= speed_duration);
    let rest_time = duration - speed_duration;
    rest_time * speed_duration
}

#[cfg(test)]
mod testing {
    use crate::day6::{simuluate_reached_distance_at, solve_task_2, NumericValue};

    use super::solve_task_1;

    #[test]
    fn should_simuluate_reached_distance_at() {
        fn assert_case(
            duration: NumericValue,
            speed_duration: NumericValue,
            expected: NumericValue,
        ) {
            let actual = simuluate_reached_distance_at(duration, speed_duration);
            assert_eq!(
                expected, actual,
                "Duration: {}\n Speed Duration: {}",
                duration, speed_duration
            );
        }
        const DURATION: NumericValue = 7;

        for (speed_duration, expected) in [
            (0, 0),
            (1, 6),
            (2, 10),
            (3, 12),
            (4, 12),
            (5, 10),
            (6, 6),
            (7, 0),
        ] {
            assert_case(DURATION, speed_duration, expected)
        }
    }

    #[test]
    fn should_solve_task_1() {
        let input = include_str!("day6/day_example_input.txt");
        let actual = solve_task_1(input);
        let expected = "288";
        assert_eq!(expected, actual);
    }

    #[test]
    fn should_solve_task_2() {
        let input = include_str!("day6/day_example_input.txt");
        let actual = solve_task_2(input);
        let expected = "71503";
        assert_eq!(expected, actual);
    }
}
